--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r148
Project Date		: 20090516182656

File				: SKG_GUI_List.lua
Commit Author		: zhinjio
Commit Revision		: 118
Commit Date			: 20081030164922
************************************************************************
Description	:
	Definition of the list management frame, and all the associated
	functions

TODO		:

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]
local SKG = LibStub( "AceAddon-3.0"):GetAddon( "SKG" )
local SKGL = LibStub( "AceLocale-3.0"):GetLocale( "SKG" )

function SKG_CreateNewList( value  )
	SKG_Debug( "Creating new list: " .. value )
	if ( SKG.db.profile.RollLists[value] == nil ) then
		if ( ( value == "SKG" ) or ( value == "Players" ) or ( value == "Alts" ) ) then
			SKG_Print( SKGL["SKG_ERROR_RESERVEDLISTNAME"] )
		else
			-- this is a new list
			local newTID = SKG_CreateTID( 0, 0, 0 )
			local Payload = { newTID, value }
			SKG_SendComm( "LSTCR", Payload, "GUILD", nil )
		end
	else
		-- This list exists already. Let 'em know and exit
		SKG_Print( SKGL["SKG_ERROR_LISTEXISTS"] )
	end
end

function SKG_DeleteList()
	local value = SKG_ActiveListTitle:GetText()
	if ( value == nil ) then
		-- no list currently selected. flag error
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
	else 
		SKG_Debug( "Removing list: " .. value )
		if ( SKG.db.profile.RollLists[value] == nil ) then
			-- list doesn't exist. Flag an error
			SKG_Print( SKGL["SKG_ERROR_LISTDOESNTEXIST"] )
		else
			-- This list exists, nuke it.
			local newTID = SKG_CreateTID( 0, 0, 0 )
			local Payload = { newTID, value }
			SKG_SendComm( "LSTRM", Payload, "GUILD", nil )
		end
	end
end

function SKG_RenameList( value )
	-- renames current list to "value"
	local currlist = SKG_ActiveListTitle:GetText()
	if ( currlist == nil ) then
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
	else
		SKG_Debug( "Renaming list: " .. currlist .. " to " .. value )
		if ( SKG.db.profile.RollLists[currlist] == nil ) then
			-- list doesn't exist. Flag an error
			SKG_Print( SKGL["SKG_ERROR_LISTDOESNTEXIST"] )
		else
			if ( SKG.db.profile.RollLists[value] == nil ) then
				-- Go ahead, target name is not in use
				local newTID = SKG_CreateTID( 0, 0, 0 )
				local Payload = { newTID, currlist, value }
				SKG_SendComm( "LSTRN", Payload, "GUILD", nil )
			else
				SKG_Print( SKGL["SKG_ERROR_LISTEXISTS"] )
			end
		end
	end
end

function SKG_CloneList( value )
	-- clones current list to "value"
	local currlist = SKG_ActiveListTitle:GetText()
	if ( currlist == nil ) then
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
	elseif ( currlist == "Players" ) then
		SKG_Print( SKGL["SKG_ERROR_RESERVEDLIST"] )
	else
		SKG_Debug( "Cloning list: " .. currlist .. " to " .. value )
		if ( SKG.db.profile.RollLists[currlist] == nil ) then
			-- list doesn't exist. Flag an error
			SKG_Print( SKGL["SKG_ERROR_LISTDOESNTEXIST"] )
		else
			if ( SKG.db.profile.RollLists[value] == nil ) then
				-- Go ahead, target name is not in use
				local newTID = SKG_CreateTID( 0, 0, 0 )
				local Payload = { newTID, currlist, value }
				SKG_SendComm( "LSTCL", Payload, "GUILD", nil )
			else
				SKG_Print( SKGL["SKG_ERROR_LISTEXISTS"] )
			end
		end
	end
end

function SKG_SnapshotList()
	-- Payload: { TID, ListName, ListHash }
	local currList = SKG_ActiveListTitle:GetText()
	if ( currList == nil ) then
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
		return
	end
	if ( currList == "Players" ) then
		SKG_Print( SKGL["SKG_ERROR_INVALIDLIST"] )
		return
	end
	-- create TID, get hashes, and send out
	local newTID = SKG_CreateTID( 0, 0, 0 )
	local db = SKG.db.profile
	local latestTID = SKG_GetLatestTID( db.Log[currList] )
	local listHash, shortHash = SKG_HashRollList( latestTID, currList )
	local Payload = { newTID, currList, listHash }

	SKG_SendComm( "SYNSS", Payload, "GUILD", nil )
end

function SKG_BroadcastList()
	-- This will send a LSTBD (list broadcast) for the current list
	-- Only "U" level users will pay attention to the packet
	local currlist = SKG_ActiveListTitle:GetText()
	local Payload = {}
	if ( currlist == nil ) then
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
		return
	elseif ( currlist == "Players" ) then
		-- So, actually, I do need to be able to broadcast the players list
		Payload = {
			currlist,
			SKG.db.profile.Players
		}
	else
		-- any other qualifiers? I won't even have this panel unless
		-- I'm a syncer...
		-- I *can* send an empty list if thats what it should be 
		Payload = {
			currlist,
			SKG.db.profile.RollLists[currlist]
		}
	end
	SKG_SendComm( "LSTBD", Payload, "GUILD", nil )
	SKG_Print( SKGL["SKG_MSG_LISTBROADCAST"] )
end

function SKG_SpamList()
	local currlist = SKG_ActiveListTitle:GetText()
	if ( currlist == nil ) then
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
	elseif ( currlist == "Players" ) then
		SKG_Print( SKGL["SKG_ERROR_RESERVEDLIST"] )
	else
		-- we shouldn't be able to reach this code unless we're in a raid,
		-- so don't bother checking...

		local db = SKG.db.profile
		local pList = db.Players
		local rList = db.RollLists[currlist]
		local outList = {}
		-- loop through. Flag anyway who is currently a raider, or is currently
		-- reserved for output.

		for i = 1, #rList do
			local tClass = ""
			local tName = ""
			local found = false
			for j = 1, #SKG.Raiders do
				if ( rList[i] == SKG.Raiders[j].mPID ) then
					tClass = SKG.Raiders[j].class
					tName = SKG.Raiders[j].name
					found = true
					break
				end
			end
			if ( found == false ) then
				for j = 1, #SKG.ReservedPlayers do
					if ( rList[i] == SKG.ReservedPlayers[j] ) then
						-- these must be mains, I think
						tClass = pList[SKG.ReservedPlayers[j]].class
						tName = pList[SKG.ReservedPlayers[j]].name
						found = true
						break
					end
				end
			end
			if ( found == true ) then
				local t = {
					name = tName,
					class = tClass,
					pos = i
				}
				tinsert( outList, t )
			end
		end
		-- Spit it out.
		if ( next( outList ) == nil ) then
			-- this should never happen. Just leave this blank for now
		else
			SendChatMessage( string.format( SKGL["SKG_MSG_SPAMHDR"], currlist ) , "RAID" )
			local fiver = 0
			local outString = ""
			for i = 1, #outList do
				if ( fiver == 0 ) then
					outString = "SKG: " .. outList[i].name .. "(" .. outList[i].pos .. ")"
				else
					outString = outString .. ", " .. outList[i].name .. "(" .. outList[i].pos .. ")"
				end
				fiver = fiver + 1
				if ( fiver == 5 ) then
					SendChatMessage( outString, "RAID" )
					fiver = 0
				end
			end
			if ( fiver ~= 0 ) then
				SendChatMessage( outString, "RAID" )
			end
		end
	end	
end

function SKG_UpdateCurrentList()
	-- this routine is going to get called whenever something changes that potentially
	-- means a change in the way a list is going to be displayed. This could mean
	-- a change in the list itself, or an option change that changes the way a list
	-- should be displayed. This also manages our FauxScrollFrame stuff. As such, the
	-- code should go in three stages:
	-- 1) recalculate the elements of the current should be displayed, and stuff the
	--		displayed values into a table (displayTable) (this bit should be sensitive
	--		to whether or not we're in the Players list or not)
	-- 2) called FauxScrollFrame_Update with the new parameters
	-- 3) Scroll through our buttons and spit out displayTable

	local currList = SKG_ActiveListTitle:GetText()
	if ( currList == nil ) then
		-- do nothing, we're not looking at a list atm
	else
	
		-- Stage 1

		-- Clear out the current list
		for i = 1,SKG_maxVisibleListItems do
			local li = SKG_ListItem[i]
			li:SetText( "" )
			li._PID = 0
			SKG_TooltipDisplay( li, "", 0 )
			-- reset to unselected textures
			li:SetNormalTexture( "" )
			li:SetHighlightTexture( "" )
		end

		-- Setup locals
		local displayTable = {}
		local db = SKG.db.profile
		local pList = db.Players
		local aList = db.Alts
		-- display table should include the following entries to make things easy:
		--	{
		--		color = { r=x, g=y, b=z },
		--		text = "Some text value",
		--		PID = playerPID,					-- used for hidden variable
		--		tooltip = "SomeToolTipString"
		--	}
		if ( currList == "Players" ) then
			if ( next( pList ) ~= nil ) then
				-- since we want to use the sorted list, use SPlayers
				for i = 1, #SKG.SPlayers do
					local currPID = SKG.SPlayers[i]
					local pName = pList[currPID].name
					local pClass = pList[currPID].class
					local pColor = { r = .1, g = .4, b = 1 }
					local TT = ""
					-- build altlist for tooltip
					if ( aList[currPID] ) then
						TT = SKGL["SKG_ALTS_TT"]
						for j = 1, #aList[currPID] do
							altName = aList[currPID][j].name
							altClass = aList[currPID][j].class
							TT = TT .. "    |c" .. RAID_STRING_COLORS[altClass] .. " " .. altName .. "|r\n"
						end
					else
	 					TT = SKGL["SKG_NOALTS_TT"]
					end
					local t = {
						color = pColor,
						text = pName,
						PID = currPID,
						tooltip = TT
					}
					tinsert( displayTable, t )
				end
			end
			-- Update the synckey on the SyncFrame
			local listHash, shortHash = SKG_HashRollList( 0, "Players" )
			SKG_SYF_MyKeyText:SetText( listHash .. shortHash )
		else
			local rList = db.RollLists[currList]

			for i = 1, #rList do
				local currPID = rList[i]
				local displayName = ""
				local displayClass = ""
				local displayColor = {}
				local TT = ""
				local isReserved = false
				-- isInRaid will have one of three value
				--		0 - we're not in a raid at all
				--		1 - we're in a raid, but this toon isn't with us
				--		2 - we're in a raid, and this toon IS with us
				local isInRaid = 0 
				local willDisplay = false

				-- First, if we've somehow managed to get an entry into this
				-- Player's list, tell the user so he can go sync that and not
				-- get further errors
				if not ( pList[currPID] ) then
					if ( SKG_SECURITY_LEVEL ~= U ) then
						SKG_Print( SKGL["SKG_ERROR_MISSINGPLAYERS"] )
					else
						SKG_Print( SKGL["SKG_ERROR_MISSINGPLAYERS2"] )
					end
					return
				end
				-- If we're in a raid, we may need to display an Alt's name
				-- instead. Go through and check for that. While we're at
				-- it, if we find the person in the current raid, flag that
				-- we found them
				if ( SKG.inRaid ) then
					isInRaid = 2
					-- is this person (or one of his alts) in our raid?
					for j = 1, #SKG.Raiders do
						if ( SKG.Raiders[j].mPID == currPID ) then
							-- If the player is an alt, indicate that in the display name
							-- jira SKG-118
							if ( SKG.Raiders[j].isAlt ) then
								displayName = SKG.Raiders[j].name .. " (A)"
								-- Also, make the tooltip show the player's main's name
								TT = SKGL["SKG__MAIN"] .. ": " ..  pList[currPID].name
							else
								displayName = SKG.Raiders[j].name
							end
							displayClass = SKG.Raiders[j].class
							isInRaid = 1
							willDisplay = true
							break
						end
					end
				else
					-- If we're not in a raid, we always display, no matter
					-- what the settings are
					displayName = pList[currPID].name
					displayClass = pList[currPID].class
					willDisplay = true
				end

				-- even if we didn't find them in the current raid, they
				-- might be reserved
				for j = 1, #SKG.ReservedPlayers do
					if ( currPID == SKG.ReservedPlayers[j] ) then
						displayName = pList[currPID].name
						displayClass = pList[currPID].class
						isReserved = true
						willDisplay = true
						isInRaid = 2
						break
					end
				end

				-- If our "raidonly" flag isn't set, we always display
				if ( db.Options.General.raidonly ~= true ) then
					-- If we don't already have a different displayName from
					-- being in a raid with an Alt, set displayName with the 
					-- player's name
					if ( displayName == "" ) then
						displayName = pList[currPID].name
						displayClass = pList[currPID].class
					end
					willDisplay = true
				end

				if ( willDisplay == true ) then
					-- we've met whatever conditions we need to. Make it so
					if db.Options.General.classColors then
						displayColor = {
							r = RAID_CLASS_COLORS[displayClass].r, 
							g = RAID_CLASS_COLORS[displayClass].g, 
							b = RAID_CLASS_COLORS[displayClass].b
						}
					else
						displayColor = { r = .1, g = .4, b = 1 }
					end
					
					-- if we made it this far, but we're not in the raid, turn the color
					-- to grey
					if ( isInRaid == 2 ) then
						displayColor = { r = .5, g = .5, b = .5 }
					end

					if ( isReserved == true ) then
						displayName = i .. ": *R* " .. displayName 
					else
						displayName = i .. ": " .. displayName
					end

					-- If we're not a user, provide "Last activity" tooltip
					if ( ( SKG_SECURITY_LEVEL ~= "U" ) and ( TT == "" ) )  then
						local lclEntry = db.LastMod[currList][currPID]
						local timeSince = SKG_TimeSince( lclEntry )
						TT = SKGL["SKG_ACTIVITY_TT"] .. timeSince
					end

					local t = {
						color = displayColor,
						text = displayName,
						PID = currPID,
						tooltip = TT
					}
					tinsert( displayTable, t )
				end
			end
			-- Update the synckey on the SyncFrame, but only if we're an admin
			if ( SKG_SECURITY_LEVEL ~= "U" ) then
				local latestTID = SKG_GetLatestTID( db.Log[currList] )
				local listHash, shortHash = SKG_HashRollList( latestTID, currList )
				SKG_SYF_MyKeyText:SetText( shortHash )
			end
		end

		-- Stage 2
		local entries = #displayTable
		FauxScrollFrame_Update( SKG_ScrollFrame, entries, SKG_maxVisibleListItems, 16 );

		-- Stage 3
		local listOffset = FauxScrollFrame_GetOffset(SKG_ScrollFrame)
		local buttonIndex = 1
		local dtIndex = buttonIndex + listOffset
		while ( ( dtIndex <= #displayTable ) and ( buttonIndex <= SKG_maxVisibleListItems ) ) do
			local clr = displayTable[dtIndex].color
			local txt = displayTable[dtIndex].text
			local pID = displayTable[dtIndex].PID
			local TT = displayTable[dtIndex].tooltip

			SKG_ListItem[buttonIndex].text:SetTextColor( clr.r, clr.g, clr.b )
			SKG_ListItem[buttonIndex].text:SetText( txt )
			SKG_ListItem[buttonIndex]._PID = pID
			SKG_TooltipDisplay( SKG_ListItem[buttonIndex], TT, 0 )

			-- Check if this is the currently "selected" item
			local li = SKG_ListItem[buttonIndex]
			if ( pID == SKG.selectedPID ) then
				SKG_ScrollFrame.selected = buttonIndex + FauxScrollFrame_GetOffset( SKG_ScrollFrame )
				-- select
				li:SetNormalTexture( [[Interface\Addons\SKG\BLP\highlight]] )
				local nTex = li:GetNormalTexture()
				nTex:SetAllPoints()
				nTex:SetTexCoord( (30/256), (1-(10/256)), (6/32), (1-(6/32)) )
				li:SetHighlightTexture( [[Interface\Addons\SKG\BLP\highlight]] )
				local hTex = li:GetHighlightTexture()
				hTex:SetAllPoints()
				hTex:SetTexCoord( (30/256), (1-(10/256)), (6/32), (1-(6/32)) )
			end

			buttonIndex = buttonIndex + 1
			dtIndex = dtIndex + 1
		end
	end
end

function SKG_InsertPlayer()
	-- We don't do anything unless we've got a list to insert into...
	local currList = SKG_ActiveListTitle:GetText()
	
	local lClass = ""
	local enClass = ""
	local tName = ""
	local timeStamp = ""
	local doInsert = false

	SKG_Debug( "in SKG_InsertPlayer" )
	if ( currList == nil ) then
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
	else
		if ( doInsert ) then SKG_Debug( "doInsert = true" ) else SKG_Debug( "doInsert = false" ) end
		-- Check to see if we have someone targetted. If so, we use them.
		-- If not, we have to query for name and class
		if UnitExists( "target" ) then
			-- ok, we've got someone targetted. Awesome
			-- Make sure they are a player, and friendly to us. Otherwise, flag errors.
			if UnitIsPlayer( "target" ) then
				if UnitIsFriend( "target", "player" ) then
					-- GTG. Get our data and stuff it in
					lClass, enClass = UnitClass( "target" )
					tName = UnitName( "target" )
				else
					SKG_Print( SKGL["SKG_ERROR_NOTFRIENDLY"] )
					return
				end
			else
				SKG_Print( SKGL["SKG_ERROR_NOTPLAYER"] )
				return
			end
		end

		-- Are we here as the result of a frame submit already?
		if ( SKG.formSubmit == false) then
			if ( currList == "Players" ) then
				-- Show the input frame with everything we know filled in. We
				-- want to give them the opportunity to tell us it is an Alt
				SKG_LFIPF_EditBox:SetText( tName )
				if not ( lClass == "" ) then
					for index = 1, #SKG_ClassNames do
						if ( enClass == SKG_enClassNames[index] ) then
							SKG.selectedClass = index
							SKG_LFIPF_ClassChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTCLASS_TEXT"] .. ": " .. SKG_ClassNames[index] )
						end
					end
				end
				SKG_LF_IPFrame:Show()
				return
			else
				-- We're in a raid list. Just show the list of toons to select
				-- from
				SKG_LF_TCFrame:Show()
				return
			end
			doInsert = false
		else
			-- Yup! Assign out values and do it
			tName = SKG.selectedToon
			enClass = SKG_enClassNames[SKG.selectedClass]
			SKG.formSubmit = false
			SKG.selectedClass = 0
			doInsert = true
		end

		-- cap the first letter
		tName:gsub( "(.)", string.upper, 1 )

		-- Check if the person already exists in the current list
		-- Possible insertions:
		--		List = Players
		--			Main (cH = tName, list = Players)
		--			Alt (cH = tName, list = Alts)
		--		List = RollList
		--			Main (cH = tName, list = currList)
		--			Alt (cH = mainPID, list = currList)
		--
		local checkHash = ""
		local pList = SKG.db.profile.Players
		local aList = SKG.db.profile.Alts
		if ( currList == "Players" ) then
			SKG_Debug("list is Players")
			-- First check if this person's been inserted as a main anywhere
			checkHash = SKG_HashName( tName )
			if ( pList[checkHash] ) then
				SKG_Print( SKGL["SKG_ERROR_DUPLICATEPLAYER"] )
				return
			end
			-- Now check if they're anywhere listed as an alt
			for hash, altList in pairs( aList ) do
				for i, altrec in pairs( aList[hash] ) do
					if ( altrec.name == tName ) then
						SKG_Print( SKGL["SKG_ERROR_DUPLICATEPLAYER"] )
						return
					end
				end
			end
		else
			SKG_Debug("list is not Players")
			SKG_Debug( "mainPID = " .. SKG.mainPID )
			SKG_Debug( "---name = " .. pList[SKG.mainPID].name )
			SKG_Debug( "---class = " .. pList[SKG.mainPID].class )
			if ( SKG.isAlt == true ) then
				SKG_Debug("isAlt is true")
				checkHash = SKG_HashName( pList[SKG.mainPID].name )
			else
				SKG_Debug("isAlt is not true")
				checkHash = SKG_HashName( tName )
			end
			SKG_Debug( "checkhash = " .. checkHash )
			local rList = SKG.db.profile.RollLists[currList]
			for index, plPID in pairs( rList ) do
				if ( plPID == checkHash ) then
					SKG_Print( SKGL["SKG_ERROR_DUPLICATEPLAYER"] )
					return
				end
			end
		end

		if ( doInsert ) then
			local newTID = SKG_CreateTID( 0, 0, 0 )
			local Payload = {}

			if ( currList == "Players" ) then
				-- Payload: TID Player Class [PID]
				if ( SKG.isAlt == true ) then
					Payload = { newTID, tName, enClass, SKG.mainPID }
				else 
					Payload = { newTID, tName, enClass }
				end
				SKG_SendComm( "PLRAD", Payload, "GUILD", nil )
			else
				-- Payload: TID List PID [Position]
				-- Auto-Add mode?
				if ( SKG.db.profile.Options.List.AutoAdd == true ) then
					local position = 0
					local dblist = SKG.db.profile.RollLists[currList]
					if ( #dblist == 0 ) then
						position = 1
					else
						position = math.random( 1, #dblist )
					end
					SKG_Debug( "autoadd position = " .. position )
					Payload = { newTID, currList, checkHash, position }
				else
					Payload = { newTID, currList, checkHash }
				end
				SKG_SendComm( "PLRIN", Payload, "GUILD", nil )
			end
		end
	end
end

function SKG_MovePlayer( direction )
	SKG_Debug( direction )
	-- direction will be 1 for up, 0 for down
	local ActiveList = SKG_ActiveListTitle:GetText()
	if ( ActiveList == nil ) then
		-- No active list
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
	else
		-- list active, anything selected?
		if ( SKG_ScrollFrame.selected == 0 ) then
			-- Can't move someone up if noone's selected. Say so
			SKG_Print( SKGL["SKG_ERROR_NOPLAYERSELECTED"] )
		else
			local newTID = SKG_CreateTID( 0, 0, 0 )
			local currPID = SKG.selectedPID

			local rList = SKG.db.profile.RollLists[ActiveList]
			local Payload = {}
			if ( direction == 1 ) then
				if ( rList[1] == currPID ) then
					-- You can't move the top player up. Say so
					SKG_Print( SKGL["SKG_ERROR_CANTMOVEHIGHER"] )
				else
					Payload = { newTID, ActiveList, currPID, "UP" }
					SKG_SendComm( "PLRMV", Payload, "GUILD", nil )
				end
			else
				local numPlayers = #rList
				if ( rList[numPlayers] == currPID ) then
					-- You can't move the bottom player down. Say so
					SKG_Print( SKGL["SKG_ERROR_CANTMOVELOWER"] )
				else
					Payload = { newTID, ActiveList, currPID, "DOWN" }
					SKG_SendComm( "PLRMV", Payload, "GUILD", nil )
				end
			end
		end
	end
end

function SKG_RemovePlayer()
	local ActiveList = SKG_ActiveListTitle:GetText()
	if ( ActiveList == nil ) then
		-- No active list
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
	else
		db = SKG.db.profile
		-- list active, anything selected?
		if ( SKG_ScrollFrame.selected == 0 ) then
			-- Can't remove if noone's selected. Say so
			SKG_Print( SKGL["SKG_ERROR_NOPLAYERSELECTED"] )
		else
			local listOffset = FauxScrollFrame_GetOffset(SKG_ScrollFrame)
			local buttonIndex = SKG_ScrollFrame.selected - listOffset
			local PID = SKG_ListItem[buttonIndex]._PID
			SKG.mainPID = PID
			local selName = db.Players[PID].name
			SKG_Debug( "selected user is : " .. PID .. " : " .. selName )
			-- Is this the Players list or not?
			if ( ActiveList == "Players" ) then
				SKG_Debug( "deleting from Players" )
				-- Does this player have alts?
				if ( db.Alts[PID] == nil ) then
					-- No alts
					SKG_Debug( "no alts" )
					SKG.isAlt = false
					SKG.selectedToon = ""
					SKG_Debug( "about to confirm dialog" )
					StaticPopup_Show( "SKG_CONFIRM_PLAYER_DELETE" )
					SKG_Debug( "after confirm dialog" )
				else
					-- He's got alts, show the selector instead
					SKG_Debug( "he's got alts" )
					-- populate alt chooser (SKG.altSelections)
					SKG.altSelections.args = {}
					for index = 1, #db.Alts[PID]  do
						altName = db.Alts[PID][index].name
						altClass = db.Alts[PID][index].class
						SKG_Debug( "n= " .. altName .. " c= " .. altClass )
						local colorcode = RAID_STRING_COLORS[altClass]
						local pValue = PID .. "|" .. altName
						local t = {
							type = "execute",
							name = "|c" .. colorcode .. " " .. altName .. "|r",
							desc = "|c" .. colorcode .. " " .. altName .. "|r",
							func = function ( crapola )
								pID, pName = strsplit( "|", crapola )
								SKG.selectedToon = pName
								SKG_LFACF_AltChooser:SetText( SKGL["SKG_LBL_LFACF_SELECTTOON_TEXT"] .. ": " .. pName )
								Dewdrop:Close()
							end,
							passValue = pValue
						}
						tinsert( SKG.altSelections.args, t )
					end
					SKG_LF_ACFrame:Show()
				end
			else
				SKG_Debug( "deleting from non-Players" )
				StaticPopup_Show( "SKG_CONFIRM_PLAYER_REMOVE" )
			end
		end
	end
end

function SKG_DeletePlayer()
	-- If we're here, we've confirmed the dialog
	local ActiveList = SKG_ActiveListTitle:GetText()

	local newTID = SKG_CreateTID( 0, 0, 0 )
	local Payload = {}

	SKG_Debug( "in DeletePlayer" )
	if ( ActiveList == "Players" ) then
		SKG_Debug( "list is Players" )
		if ( SKG.isAlt == true ) then
			SKG_Debug( "isAlt" )
			Payload = { newTID, SKG.mainPID, SKG.selectedToon } 
		else
			Payload = { newTID, SKG.mainPID } 
		end
		SKG_SendComm( "PLRDL", Payload, "GUILD", nil )
	else
		SKG_Debug( "list is NOT Players" )
		Payload = { newTID, ActiveList, SKG.mainPID }
		SKG_SendComm( "PLRRM", Payload, "GUILD", nil )
	end
end

function SKG_SuicidePlayer()
	SKG_Debug( "in SuicidePlayer" )
	local ActiveList = SKG_ActiveListTitle:GetText()
	if ( ActiveList == nil ) then
		-- no list selected. error out
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
		return
	end
	
	-- Was going to add in a check to see if the selected list is the
	-- "Players" list, but at least in theory, the "Suicide Player" button
	-- should always be disabled if the Players list is the active list.

	-- list active, anything selected?
	if ( SKG_ScrollFrame.selected == 0 ) then
		-- Can't suicide someone if noone's selected. Say so
		SKG_Print( SKGL["SKG_ERROR_NOPLAYERSELECTED"] )
		return
	end
	local db = SKG.db.profile.RollLists[ActiveList]
	local currPID = SKG.selectedPID
	local newTID = SKG_CreateTID( 0, 0, 0 )
	local Payload = {}
	-- if we're in a raid, we're gonna do something a bit different than
	-- if we're not
	if ( SKG.inRaid ) then
		-- In this case, I need to generate a list of affected people. This
		-- includes anyone in the raid + anyone currently set as reserved.
		local affectedList = {}
		local selectedValid = false
		for i = 1, #SKG.Raiders do
			tinsert( affectedList, SKG.Raiders[i].mPID )
			if ( currPID == SKG.Raiders[i].mPID ) then
				selectedValid = true
			end
		end
		-- Add in any reserved players
		for i = 1, #SKG.ReservedPlayers do
			tinsert( affectedList, SKG.ReservedPlayers[i] )
			if ( SKG.ReservedPlayers[i] == currPID ) then
				selectedValid = true
			end
		end

		-- Its possible the toon they have selected is *not* currently
		-- in the raid. Thats a nono. Tell 'em so.
		if ( selectedValid == false ) then
			SKG_Print( SKGL["SKG_ERROR_BADPLAYERSELECTED"] )
			return
		end

		Payload = { newTID, ActiveList, currPID, affectedList }
		SKG_SendComm( "PLRSC", Payload, "GUILD", nil )
	else
		-- you will usually only see this if someone is doing "after the fact"
		-- manipulation. They will put all players on "Reserve" and simulate
		-- suicides to get the lists back in the right place.

		-- Make our list out of the currently selected player + anyone marked
		-- "reserved".
		-- Add in any reserved players
		local affectedList = {}
		tinsert( affectedList, currPID )
		-- add in reserved
		for i = 1, #SKG.ReservedPlayers do
			tinsert( affectedList, SKG.ReservedPlayers[i] )
		end
		Payload = { newTID, ActiveList, currPID, affectedList }
		SKG_SendComm( "PLRSC", Payload, "GUILD", nil )
	end
end

function SKG_LF_AutoAddCB_Clicked()
	-- Check auto-add mode
	local dbLO = SKG.db.profile.Options.List
	if ( dbLO.AutoAdd ) then
		dbLO.AutoAdd = false
		SKG_LF_AutoAddCB:SetChecked( false )
	else
		dbLO.AutoAdd = true
		SKG_LF_AutoAddCB:SetChecked( true )
	end
end

function SKG_ReservePlayer()
	SKG_Debug( "In ReservePlayer" )

	local ActiveList = SKG_ActiveListTitle:GetText()
	if ( ActiveList == nil ) then
		-- No active list
		SKG_Print( SKGL["SKG_ERROR_NOLISTSELECTED"] )
	else
		if ( ActiveList == "Players" ) then
			-- Shouldn't be able to get here
		else
			-- list active, anything selected?
			if ( SKG_ScrollFrame.selected == 0 ) then
				-- Can't reserve someone if noone's selected. Say so
				SKG_Print( SKGL["SKG_ERROR_NOPLAYERSELECTED"] )
			else
				local currPID = SKG.selectedPID
				-- see if they're already reserved
				local notfound = true
				local i = 1
				while ( ( i <= #SKG.ReservedPlayers ) and ( notfound == true ) ) do
					if ( currPID == SKG.ReservedPlayers[i] ) then
						notfound = false
					end
					i = i + 1
				end
				if ( notfound == true ) then
					-- reserve
					tinsert( SKG.ReservedPlayers, currPID )
					SKG_LF_ReservePlayerButton:SetText( SKGL["SKG_LBL_LF_UNRESERVEPLAYER_TEXT"] )
				else
					-- unreserve
					i = i - 1
					tremove( SKG.ReservedPlayers, i )
					SKG_LF_ReservePlayerButton:SetText( SKGL["SKG_LBL_LF_RESERVEPLAYER_TEXT"] )
					-- if we're in raidonly mode, deselect
					if ( SKG.db.profile.Options.General.raidonly ) then
						SKG_ScrollFrame.selected = 0
						SKG.selectedPID = 0
						SKG_LF_ReservePlayerButton:Disable()
					end
				end
				SKG_UpdateCurrentList()
			end
		end
	end
end

function SKG_FindMissing()
	-- this only works if we're in a raid
	local outString = ""
	if ( SKG.inRaid == true ) then
		local currList = SKG_ActiveListTitle:GetText()
		if ( currList == "Players" ) then
			local missingPlayers = ""
			for i = 1, #SKG.Raiders do
				if ( SKG.Raiders[i].mPID == 0 ) then
					local c = SKG.Raiders[i].class
					local n = SKG.Raiders[i].name
					missingPlayers = missingPlayers .. " |c" .. RAID_STRING_COLORS[c] .. " " .. n .. "|r"
				end
			end
			if ( missingPlayers == "" ) then
				outString = SKGL["SKG_MSG_NONEPLAYERS"]
			else
				outString = SKGL["SKG_MSG_MISSINGPLAYERS"]
				outString = outString .. missingPlayers .. "\n"
				outString = outString .. SKGL["SKG_MSG_CLICKADDMISSING"]
			end
		else
			local missingPlayers = ""
			for i = 1, #SKG.Raiders do
				if ( SKG.Raiders[i].mPID == 0 ) then
					local c = SKG.Raiders[i].class
					local n = SKG.Raiders[i].name
					missingPlayers = missingPlayers .. " |c" .. RAID_STRING_COLORS[c] .. " " .. n .. "|r"
				end
			end
			if ( missingPlayers == "" ) then
				local listMissing = ""
				local db = SKG.db.profile
				local pList = db.Players
				local rList  = db.RollLists[currList]
				for i = 1, #SKG.Raiders do
					currPID = SKG.Raiders[i].mPID
					if ( currPID ~= 0 ) then
						-- They're already in the Players list, are they in this list?
						local notfound = true
						for j = 1, #rList do
							if ( rList[j] == SKG.Raiders[i].mPID ) then
								notfound = false
								break
							end
						end
						if ( notfound == true ) then
							local c = pList[currPID].class
							local n = pList[currPID].name
							listMissing = listMissing .. " |c" .. RAID_STRING_COLORS[c] .. " " .. n .. "|r"
						end
					end
				end
				if ( listMissing == "" ) then
					outString = SKGL["SKG_MSG_NONEMISSING"]
				else
					outString = SKGL["SKG_MSG_LISTMISSING"]
					outString = outString .. listMissing .. "\n"
					outString = outString .. SKGL["SKG_MSG_CLICKADDMISSING"]
				end
			else
				outString = SKGL["SKG_MSG_MISSINGPLAYERS"]
				outString = outString .. missingPlayers .. "\n"
				outString = outString .. SKGL["SKG_MSG_SWITCHPLAYERS"]
			end
		end
	else
		outString = SKGL["SKG_ERROR_NOTINRAID"]
	end
	SKG_Print( outString )
end

function SKG_AddMissing()
	-- this only works if we're in a raid
	local outString = ""
	if ( SKG.inRaid == true ) then
		local currList = SKG_ActiveListTitle:GetText()
		if ( currList == "Players" ) then
			for i = 1, #SKG.Raiders do
				if ( SKG.Raiders[i].mPID == 0 ) then
					StaticPopup_Show( "SKG_CONFIRM_PLAYER_ADDMISSING" )
					return
				end
			end
			SKG_Print( SKGL["SKG_MSG_NONEPLAYERS"] )
			return
		else
			local missingPlayers = ""
			for i = 1, #SKG.Raiders do
				if ( SKG.Raiders[i].mPID == 0 ) then
					local c = SKG.Raiders[i].class
					local n = SKG.Raiders[i].name
					missingPlayers = missingPlayers .. " |c" .. RAID_STRING_COLORS[c] .. " " .. n .. "|r"
				end
			end
			if ( missingPlayers == "" ) then
				-- iterate over list, already in Player's list. don't care about main/alt
				local listMissing = ""
				local db = SKG.db.profile
				local pList = db.Players
				local rList  = db.RollLists[currList]
				for i = 1, #SKG.Raiders do
					currPID = SKG.Raiders[i].mPID
					if ( currPID ~= 0 ) then
						-- They're already in the Players list, are they in this list?
						local notfound = true
						for j = 1, #rList do
							if ( rList[j] == SKG.Raiders[i].mPID ) then
								notfound = false
								break
							end
						end
						if ( notfound == true ) then
							StaticPopup_Show( "SKG_CONFIRM_ROLLLIST_ADDMISSING" )
							return
						end
					end
				end
				SKG_Print( SKGL["SKG_MSG_NONEMISSING"] )
				return
			else
				outString = SKGL["SKG_MSG_MISSINGPLAYERS"]
				outString = outString .. missingPlayers .. "\n"
				outString = outString .. SKGL["SKG_MSG_SWITCHPLAYERS"]
				SKG_Print( outString )
				return
			end
		end
	else
		SKG_Print( SKGL["SKG_ERROR_NOTINRAID"] )
	end
end



-- List Frame Creation

SKG_ListFrame = CreateFrame( "Frame", "SKG_ListFrame", SKG_Frame )
	SKG_ListFrame:SetClampedToScreen( true )
	SKG_ListFrame:SetHeight( 442 )
	SKG_ListFrame:SetWidth( 200 )
	SKG_ListFrame:SetPoint( "TOPLEFT", SKG_Frame, "TOPRIGHT", -5, -50 )
	SKG_ListFrame:EnableMouse( true )
	SKG_ListFrame:SetMovable( true )
	SKG_ListFrame:SetFrameLevel( 1 )
	SKG_ListFrame:SetFrameStrata( "BACKGROUND" )
	SKG_ListFrame:SetHitRectInsets( 5, 5, 5, 5 )
	SKG_ListFrame:Hide()

	local SKG_LF_Texture = SKG_ListFrame:CreateTexture( "SKG_LF_Texture", "ARTWORK" )
		SKG_LF_Texture:SetTexture( [[Interface\Addons\SKG\BLP\ListFrame_Texture]] )
		SKG_LF_Texture:SetAllPoints( SKG_ListFrame )
		SKG_LF_Texture:SetTexCoord( (( 512 - 200 ) / 512 ), 1, 0, ( 442 / 512 ) )

	local SKG_LF_TitleText = SKG_ListFrame:CreateFontString( "SKG_LF_TitleText", "OVERLAY", "GameFontHighlight" )
		SKG_LF_TitleText:SetText( SKGL["SKG_LBL_LF_TITLE_TEXT"] )
		SKG_LF_TitleText:SetPoint( "TOP", SKG_ListFrame, "TOP", 0, -3 )
		SKG_LF_TitleText:SetJustifyH( "CENTER" )
		SKG_LF_TitleText:SetHeight( 14 )
		SKG_LF_TitleText:SetWidth( 200 )

	local SKG_LF_AutoAddCB = CreateFrame( "CheckButton", "SKG_LF_AutoAddCB", SKG_ListFrame, "UICheckButtonTemplate" )
		SKG_LF_AutoAddCB:SetHeight( 24 )
		SKG_LF_AutoAddCB:SetWidth( 24 )
		SKG_LF_AutoAddCB:SetPoint( "TOPLEFT", SKG_ListFrame, "TOPLEFT", 40, -30 )
		SKG_LF_AutoAddCBText:SetText( SKGL["SKG_LBL_LF_AUTOADDCB_TEXT"] )

		SKG_LF_AutoAddCB:SetScript( "OnClick", SKG_LF_AutoAddCB_Clicked )

		SKG_TooltipDisplay( SKG_LF_AutoAddCB, "SKG_LF_AUTOADDCB_TT", 1 )

	local SKG_LF_NewListButton = SKG_CreateButton( "SKG_LF_NewListButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_ListFrame, "TOPLEFT", 15, -68, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_NEWLIST_TEXT"], "CENTER", "SKG_LF_NEWLIST_TT" )

		SKG_LF_NewListButton:SetScript( "OnClick",
			function ()
				StaticPopup_Show("SKG_NEWLIST")
			end )

	local SKG_LF_DelListButton = SKG_CreateButton( "SKG_LF_DelListButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_NewListButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_DELLIST_TEXT"], "CENTER", "SKG_LF_DELLIST_TT" )

		SKG_LF_DelListButton:SetScript( "OnClick",
			function ()
				StaticPopup_Show("SKG_CONFIRMDELETION")
			end )
	
	local SKG_LF_RenListButton = SKG_CreateButton( "SKG_LF_RenListButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_DelListButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_RENLIST_TEXT"], "CENTER", "SKG_LF_RENLIST_TT" )

		SKG_LF_RenListButton:SetScript( "OnClick",
			function ()
				StaticPopup_Show("SKG_RENAMELIST")
			end )

	local SKG_LF_CloneListButton = SKG_CreateButton( "SKG_LF_CloneListButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_RenListButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_CLONELIST_TEXT"], "CENTER", "SKG_LF_CLONELIST_TT" )

		SKG_LF_CloneListButton:SetScript( "OnClick",
			function ()
				StaticPopup_Show("SKG_CLONELIST")
			end )

	local SKG_LF_SnapshotListButton = SKG_CreateButton( "SKG_LF_SnapshotListButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_CloneListButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_SNAPLIST_TEXT"], "CENTER", "SKG_LF_SNAPLIST_TT" )

		SKG_LF_SnapshotListButton:SetScript( "OnClick",
			function ()
				StaticPopup_Show("SKG_CONFIRM_SNAPSHOT")
			end )

	local SKG_LF_BroadcastListButton = SKG_CreateButton( "SKG_LF_BroadcastListButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_SnapshotListButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_BROADCAST_TEXT"], "CENTER", "SKG_LF_BROADCAST_TT" )

		SKG_LF_BroadcastListButton:SetScript( "OnClick",
			function ()
				SKG_BroadcastList()
			end )

	local SKG_LF_SpamListButton = SKG_CreateButton( "SKG_LF_SpamListButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_BroadcastListButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_SPAMLIST_TEXT"], "CENTER", "SKG_LF_SPAMLIST_TT" )

		SKG_LF_SpamListButton:SetScript( "OnClick",
			function ()
				SKG_SpamList()
			end )

	local SKG_LF_MoveUpButton = SKG_CreateButton( "SKG_LF_MoveUpButton", SKG_ListFrame, 21, 80,
		"TOPLEFT", SKG_LF_SpamListButton, "BOTTOMLEFT", 0, -10, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_MOVEUP_TEXT"], "CENTER", "SKG_LF_MOVEUP_TT" )
--		SKG_LF_MoveUpButton:SetFont( [[Fonts\ARIALN.TTF]], 11 )

		SKG_LF_MoveUpButton:SetScript( "OnClick", 
			function ()
				SKG_MovePlayer( 1 )
			end )

	local SKG_LF_MoveDownButton = SKG_CreateButton( "SKG_LF_MoveDownButton", SKG_ListFrame, 21, 80,
		"TOPRIGHT", SKG_LF_SpamListButton, "BOTTOMRIGHT", 0, -10, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_MOVEDOWN_TEXT"], "CENTER", "SKG_LF_MOVEDOWN_TT" )
--		SKG_LF_MoveDownButton:SetFont( [[Fonts\ARIALN.TTF]], 11 )

		SKG_LF_MoveDownButton:SetScript( "OnClick", 
			function ()
				SKG_MovePlayer( 0 )
			end )

	local SKG_LF_InsertPlayerButton = SKG_CreateButton( "SKG_LF_InsertPlayerButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_MoveUpButton, "BOTTOMLEFT", 0, -10, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_INSERTPLAYER_TEXT"], "CENTER", "SKG_LF_INSPLAYER_TT" )

		SKG_LF_InsertPlayerButton:SetScript( "OnClick", SKG_InsertPlayer )

	local SKG_LF_RemovePlayerButton = SKG_CreateButton( "SKG_LF_RemovePlayerButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_InsertPlayerButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_REMOVEPLAYER_TEXT"], "CENTER", "SKG_LF_REMPLAYER_TT" )

		SKG_LF_RemovePlayerButton:SetScript( "OnClick", SKG_RemovePlayer )

	local SKG_LF_SuicidePlayerButton = SKG_CreateButton( "SKG_LF_SuicidePlayerButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_RemovePlayerButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_SUICIDEPLAYER_TEXT"], "CENTER", "SKG_LF_SUICIDEPLAYER_TT" )

		SKG_LF_SuicidePlayerButton:SetScript( "OnClick", 
			function ()
				SKG_SuicidePlayer( )
			end )

	local SKG_LF_ReservePlayerButton = SKG_CreateButton( "SKG_LF_ReservePlayerButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_SuicidePlayerButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_RESERVEPLAYER_TEXT"], "CENTER", "SKG_LF_RESERVEPLAYER_TT" )

		SKG_LF_ReservePlayerButton:SetScript( "OnClick", 
			function ()
				SKG_ReservePlayer( )
			end )

	local SKG_LF_FindMissingButton = SKG_CreateButton( "SKG_LF_FindMissingButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_ReservePlayerButton, "BOTTOMLEFT", 0, -10, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_FINDMISSING_TEXT"], "CENTER", "SKG_LF_FINDMISSING_TT" )

		SKG_LF_FindMissingButton:SetScript( "OnClick", 
			function ()
				SKG_FindMissing( )
			end )

	local SKG_LF_AddMissingButton = SKG_CreateButton( "SKG_LF_AddMissingButton", SKG_ListFrame, 21, 170,
		"TOPLEFT", SKG_LF_FindMissingButton, "BOTTOMLEFT", 0, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LF_ADDMISSING_TEXT"], "CENTER", "SKG_LF_ADDMISSING_TT" )

		SKG_LF_AddMissingButton:SetScript( "OnClick", 
			function ()
				SKG_AddMissing( )
			end )

-- OnLoad stuff here

-- Small Frame to confirm Main vs. Alt selection

	SKG.altSelections = {
		type = "group",
		handler = SKG,
		args = { },
	}

function SKG_LFACF_AltCB_Clicked( )
	if ( SKG.isAlt == true ) then
		SKG_Debug( "before: Alt is true" )
	else
		SKG_Debug( "before: Alt is false" )
	end

	if ( SKG_LFACF_AltCB:GetChecked() ) then
		SKG_Debug( "set isAlt to true" )
		SKG.isAlt = true
	else		
		SKG_Debug( "set isAlt to false" )
		SKG.isAlt = false
	end
end

-- Frame definition

	SKG_LF_ACFrame = CreateFrame( "Frame", "SKG_LF_ACFrame", SKG_ListFrame )
	SKG_LF_ACFrame:SetHeight( 120 )
	SKG_LF_ACFrame:SetWidth( 256 )
	SKG_LF_ACFrame:SetPoint( "TOPLEFT", SKG_ListFrame, "TOPRIGHT", 10, 0 )
	SKG_LF_ACFrame:EnableMouse( true )
	SKG_LF_ACFrame:SetMovable( true )
	SKG_LF_ACFrame:SetFrameLevel( 1 )
	SKG_LF_ACFrame:SetFrameStrata( "BACKGROUND" )
	SKG_LF_ACFrame:SetHitRectInsets( 5, 5, 5, 5 )
	SKG_LF_ACFrame:Hide()

	local SKG_LFACF_Texture = SKG_LF_ACFrame:CreateTexture( "SKG_LFACF_Texture", "ARTWORK" )
		SKG_LFACF_Texture:SetTexture( [[Interface\Addons\SKG\BLP\GenericDialog]] )
		SKG_LFACF_Texture:SetAllPoints( SKG_LF_ACFrame )

	local SKG_LFACF_TitleText = SKG_LF_ACFrame:CreateFontString( "SKG_LFACF_TitleText", "OVERLAY", "GameFontHighlight" )
		SKG_LFACF_TitleText:SetText( SKGL["SKG_LBL_LFACF_TITLE_TEXT"] )
		SKG_LFACF_TitleText:SetPoint( "TOP", SKG_LF_ACFrame, "TOP", 0, -10 )
		SKG_LFACF_TitleText:SetJustifyH( "CENTER" )
		SKG_LFACF_TitleText:SetHeight( 18 )
		SKG_LFACF_TitleText:SetWidth( 200 )

	local SKG_LFACF_AltCB = CreateFrame( "CheckButton", "SKG_LFACF_AltCB", SKG_LF_ACFrame, "UICheckButtonTemplate" )
		SKG_LFACF_AltCB:SetHeight( 24 )
		SKG_LFACF_AltCB:SetWidth( 24 )
		SKG_LFACF_AltCB:SetPoint( "TOPLEFT", SKG_LF_ACFrame, "TOPLEFT", 15, -26 )
		SKG_LFACF_AltCBText:SetText( SKGL["SKG_LBL_LFACF_ALTCB_TEXT"] )

		SKG_LFACF_AltCB:SetScript( "OnClick", SKG_LFACF_AltCB_Clicked )

		SKG_TooltipDisplay( SKG_LFACF_AltCB, "SKG_LFACF_ALTCB_TT", 1 )

	local SKG_LFACF_AltChooser =  SKG_CreateButton( "SKG_LFACF_AltChooser", SKG_LF_ACFrame, 21, 200,
		"TOP", SKG_LF_ACFrame, "TOP", 0, -50, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LFACF_SELECTTOON_TEXT"], "CENTER", "SKG_LFACF_SELECTTOON_TT" )

		-- onclick method
		SKG_LFACF_AltChooser:SetScript( "OnClick",
			function()
				Dewdrop:Register( this,
					'children', SKG.altSelections,
					'dontHook', true,
					'point', "TOP",
					'relativePoint', "BOTTOM"
				)
				this:SetScript( "OnClick",
					function()
						if Dewdrop:IsOpen( this ) then
							Dewdrop:Close()
						else
							Dewdrop:Open( this )
						end
					end
				)
				this:GetScript( "OnClick" )()
			end
		)

	local SKG_LFACF_OK =  SKG_CreateButton( "SKG_LFACF_OK", SKG_LF_ACFrame, 21, 90,
		"TOPLEFT", SKG_LFACF_AltChooser, "BOTTOMLEFT", 0, -1, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG__SUBMIT"], "CENTER", "SKG__SUBMIT" )

		SKG_LFACF_OK:SetScript( "OnClick", function ()
			-- Validate our data
			if ( ( SKG.isAlt == true ) and ( SKG.selectedToon == "" ) ) then
				SKG_Print( SKGL["SKG_ERROR_NOPLAYERSELECTED"] )
			else
				SKG_LFACF_AltChooser:SetText( SKGL["SKG_LBL_LFACF_SELECTTOON_TEXT"] )
				SKG_LF_ACFrame:Hide()
				SKG_DeletePlayer()
			end
		end )

	local SKG_LFACF_CANCEL = SKG_CreateButton( "SKG_LFACF_CANCEL", SKG_LF_ACFrame, 21, 90,
		"TOPRIGHT", SKG_LFACF_AltChooser, "BOTTOMRIGHT", 0, -1, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG__CANCEL"], "CENTER", "SKG__CANCEL" )
		SKG_LFACF_CANCEL:SetScript( "OnClick", function ()
			SKG_LF_ACFrame:Hide()
		end )

-- -- Frame functions

-- Small Frame for selecting an existing toon from the lists of Mains/Alts
	
-- -- Frame functions
	SKG.toonSelections = {
		type = "group",
		handler = SKG,
		args = { },
	}

-- -- Frame definitions
	SKG_LF_TCFrame = CreateFrame( "Frame", "SKG_LF_TCFrame", SKG_ListFrame )
	SKG_LF_TCFrame:SetHeight( 100 )
	SKG_LF_TCFrame:SetWidth( 256 )
	SKG_LF_TCFrame:SetPoint( "TOPLEFT", SKG_ListFrame, "TOPRIGHT", 10, 0 )
	SKG_LF_TCFrame:EnableMouse( true )
	SKG_LF_TCFrame:SetMovable( true )
	SKG_LF_TCFrame:SetFrameLevel( 1 )
	SKG_LF_TCFrame:SetFrameStrata( "BACKGROUND" )
	SKG_LF_TCFrame:SetHitRectInsets( 5, 5, 5, 5 )
	SKG_LF_TCFrame:Hide()

	local SKG_LFTCF_Texture = SKG_LF_TCFrame:CreateTexture( "SKG_LFTCF_Texture", "ARTWORK" )
		SKG_LFTCF_Texture:SetTexture( [[Interface\Addons\SKG\BLP\GenericDialog]] )
		SKG_LFTCF_Texture:SetAllPoints( SKG_LF_TCFrame )

	local SKG_LFTCF_TitleText = SKG_LF_TCFrame:CreateFontString( "SKG_LFTCF_TitleText", "OVERLAY", "GameFontHighlight" )
		SKG_LFTCF_TitleText:SetText( SKGL["SKG_LBL_LFTCF_TITLE_TEXT"] )
		SKG_LFTCF_TitleText:SetPoint( "TOP", SKG_LF_TCFrame, "TOP", 0, -15 )
		SKG_LFTCF_TitleText:SetJustifyH( "CENTER" )
		SKG_LFTCF_TitleText:SetHeight( 18 )
		SKG_LFTCF_TitleText:SetWidth( 200 )

	local SKG_LFTCF_ToonChooser =  SKG_CreateButton( "SKG_LFTCF_ToonChooser", SKG_LF_TCFrame, 21, 200,
		"TOP", SKG_LFTCF_TitleText, "BOTTOM", 0, -3, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LFTCF_SELECTTOON_TEXT"], "CENTER", "SKG_LFTCF_SELECTTOON_TT" )

		-- onclick method
		SKG_LFTCF_ToonChooser:SetScript( "OnClick",
			function()
				Dewdrop:Register( this,
					'children', SKG.toonSelections,
					'dontHook', true,
					'point', "TOP",
					'relativePoint', "BOTTOM"
				)
				this:SetScript( "OnClick",
					function()
						if Dewdrop:IsOpen( this ) then
							Dewdrop:Close()
						else
							Dewdrop:Open( this )
						end
					end
				)
				this:GetScript( "OnClick" )()
			end
		)

	local SKG_LFTCF_OK =  SKG_CreateButton( "SKG_LFTCF_OK", SKG_LF_TCFrame, 21, 90,
		"TOPLEFT", SKG_LFTCF_ToonChooser, "BOTTOMLEFT", 0, -1, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG__SUBMIT"], "CENTER", "SKG__SUBMIT" )

		SKG_LFTCF_OK:SetScript( "OnClick", function ()
			-- Validate our data
			if ( SKG.selectedToon == "" ) then
				SKG_Print( SKGL["SKG_ERROR_NOPLAYERSELECTED"] )
			else
				SKG_LFTCF_ToonChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTTOON_TEXT"] )
				SKG_LF_TCFrame:Hide()
				SKG.formSubmit = true
				SKG_InsertPlayer()
			end
		end )

	local SKG_LFTCF_CANCEL = SKG_CreateButton( "SKG_LFTCF_CANCEL", SKG_LF_TCFrame, 21, 90,
		"TOPRIGHT", SKG_LFTCF_ToonChooser, "BOTTOMRIGHT", 0, -1, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG__CANCEL"], "CENTER", "SKG__CANCEL" )
		SKG_LFTCF_CANCEL:SetScript( "OnClick", function ()
			SKG_LF_TCFrame:Hide()
		end )

-- OnLoad code
	SKG_AddAlphaGroups( SKG.toonSelections )

-- Small Frame for querying the name and class (to insert into a list)

-- -- Frame functions
SKG.classSelections = {
	type = "group",
	handler = SKG,
	args = { },
}

SKG.mainSelections = {
	type = "group",
	handler = SKG,
	args = { },
}

function SKG_LFIPF_AltCB_Clicked( )
	if ( SKG.isAlt == true ) then
		SKG_Debug( "before: Alt is true" )
	else
		SKG_Debug( "before: Alt is false" )
	end

	if ( SKG_LFIPF_AltCB:GetChecked() ) then
		SKG_Debug( "set isAlt to true" )
		SKG.isAlt = true
	else		
		SKG_Debug( "set isAlt to false" )
		SKG.isAlt = false
	end
end

-- -- Frame definintions

	SKG_LF_IPFrame = CreateFrame( "Frame", "SKG_LF_IPFrame", SKG_ListFrame )
	SKG_LF_IPFrame:SetHeight( 160 )
	SKG_LF_IPFrame:SetWidth( 256 )
	SKG_LF_IPFrame:SetPoint( "TOPLEFT", SKG_ListFrame, "TOPRIGHT", 10, 0 )
	SKG_LF_IPFrame:EnableMouse( true )
	SKG_LF_IPFrame:SetMovable( true )
	SKG_LF_IPFrame:SetFrameLevel( 1 )
	SKG_LF_IPFrame:SetFrameStrata( "BACKGROUND" )
	SKG_LF_IPFrame:SetHitRectInsets( 5, 5, 5, 5 )
	SKG_LF_IPFrame:Hide()

	local SKG_LFIPF_Texture = SKG_LF_IPFrame:CreateTexture( "SKG_LFIPF_Texture", "ARTWORK" )
		SKG_LFIPF_Texture:SetTexture( [[Interface\Addons\SKG\BLP\GenericDialog]] )
		SKG_LFIPF_Texture:SetAllPoints( SKG_LF_IPFrame )

	local SKG_LFIPF_TitleText = SKG_LF_IPFrame:CreateFontString( "SKG_LFIPF_TitleText", "OVERLAY", "GameFontHighlight" )
		SKG_LFIPF_TitleText:SetText( SKGL["SKG_LBL_LFIPF_TITLE_TEXT"] )
		SKG_LFIPF_TitleText:SetPoint( "TOP", SKG_LF_IPFrame, "TOP", 0, -15 )
		SKG_LFIPF_TitleText:SetJustifyH( "CENTER" )
		SKG_LFIPF_TitleText:SetHeight( 18 )
		SKG_LFIPF_TitleText:SetWidth( 200 )

	local SKG_LFIPF_EditBox = CreateFrame( "EditBox", "SKG_LFIPF_EditBox", SKG_LF_IPFrame, "InputBoxTemplate" )
			SKG_LFIPF_EditBox:SetFontObject(ChatFontNormalSmall)
			SKG_LFIPF_EditBox:SetHeight(18)
			SKG_LFIPF_EditBox:SetWidth(190)
			SKG_LFIPF_EditBox:SetPoint( "TOP", SKG_LF_IPFrame, "TOP", 4, -35 )
			SKG_LFIPF_EditBox:SetText("")
			SKG_LFIPF_EditBox:SetJustifyH("LEFT")

	local SKG_LFIPF_ClassChooser =  SKG_CreateButton( "SKG_LFIPF_ClassChooser", SKG_LF_IPFrame, 21, 200,
		"TOP", SKG_LFIPF_EditBox, "BOTTOM", -2, -2, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LFIPF_SELECTCLASS_TEXT"], "CENTER", "SKG_LFIPF_SELECTCLASS_TT" )

		-- onclick method
		SKG_LFIPF_ClassChooser:SetScript( "OnClick",
			function()
				Dewdrop:Register( this,
					'children', SKG.classSelections,
					'dontHook', true,
					'point', "TOP",
					'relativePoint', "BOTTOM"
				)
				this:SetScript( "OnClick",
					function()
						if Dewdrop:IsOpen( this ) then
							Dewdrop:Close()
						else
							Dewdrop:Open( this )
						end
					end
				)
				this:GetScript( "OnClick" )()
			end
		)

	local SKG_LFIPF_AltCB = CreateFrame( "CheckButton", "SKG_LFIPF_AltCB", SKG_LF_IPFrame, "UICheckButtonTemplate" )
		SKG_LFIPF_AltCB:SetHeight( 24 )
		SKG_LFIPF_AltCB:SetWidth( 24 )
		SKG_LFIPF_AltCB:SetPoint( "TOPLEFT", SKG_LFIPF_ClassChooser, "BOTTOMLEFT", -2, 2 )
		SKG_LFIPF_AltCBText:SetText( SKGL["SKG_LBL_LFIPF_ALTCB_TEXT"] )

		SKG_LFIPF_AltCB:SetScript( "OnClick", SKG_LFIPF_AltCB_Clicked )

		SKG_TooltipDisplay( SKG_LFIPF_AltCB, "SKG_LFIPF_ALTCB_TT", 1 )

	-- dropdown for list of mains
	local SKG_LFIPF_MainChooser =  SKG_CreateButton( "SKG_LFIPF_MainChooser", SKG_LF_IPFrame, 21, 170,
		"TOPLEFT", SKG_LFIPF_AltCB, "BOTTOMLEFT", 30, 3, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LBL_LFIPF_SELECTMAIN_TEXT"], "CENTER", "SKG_LFIPF_SELECTMAIN_TT" )

		-- onclick method
		SKG_LFIPF_MainChooser:SetScript( "OnClick",
			function()
				Dewdrop:Register( this,
					'children', SKG.mainSelections,
					'dontHook', true,
					'point', "TOP",
					'relativePoint', "BOTTOM"
				)
				this:SetScript( "OnClick",
					function()
						if Dewdrop:IsOpen( this ) then
							Dewdrop:Close()
						else
							Dewdrop:Open( this )
						end
					end
				)
				this:GetScript( "OnClick" )()
			end
		)

	local SKG_LFIPF_OK =  SKG_CreateButton( "SKG_LFIPF_OK", SKG_LF_IPFrame, 21, 90,
		"TOPLEFT", SKG_LFIPF_ClassChooser, "BOTTOMLEFT", 0, -42, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG__SUBMIT"], "CENTER", "SKG__SUBMIT" )

		SKG_LFIPF_OK:SetScript( "OnClick", function ()
			-- Validate our data
			local validSubmit = false
			local trimName = strtrim( SKG_LFIPF_EditBox:GetText() )
			if SKG.selectedClass == 0 then
				SKG_Print( SKGL["SKG_ERROR_NOCLASSSELECTED"] )
			else
				if trimName == "" then
						SKG_Print( SKGL["SKG_ERROR_INVALIDNAME"] )
				else 
					if ( SKG.isAlt == true ) then
						if ( SKG.mainPID == "" ) then
							SKG_Print( SKGL["SKG_ERROR_NOMAINSELECTED"] )
						else
							validSubmit = true
						end
							
					else
						validSubmit = true
					end
				end
			end

			if ( validSubmit == true ) then
				-- Clear entries and Hide
				SKG_LFIPF_EditBox:ClearFocus()
				SKG_LFIPF_EditBox:SetText( "" )
				SKG_LFIPF_ClassChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTCLASS_TEXT"] )
				SKG_LFIPF_AltCB:SetChecked( false )
				SKG_LFIPF_MainChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTMAIN_TEXT"] )
				SKG_LF_IPFrame:Hide()
				-- Make sure our variables are set properly and do your stuff
				SKG.formSubmit = true
				SKG.selectedToon = trimName
				SKG_InsertPlayer()
			end
		end )

	local SKG_LFIPF_CANCEL = SKG_CreateButton( "SKG_LFIPF_CANCEL", SKG_LF_IPFrame, 21, 90,
		"TOPRIGHT", SKG_LFIPF_ClassChooser, "BOTTOMRIGHT", 0, -42, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG__CANCEL"], "CENTER", "SKG__CANCEL" )

		SKG_LFIPF_CANCEL:SetScript( "OnClick", function ()
			SKG_LFIPF_EditBox:ClearFocus()
			SKG_LF_IPFrame:Hide()
			-- Clear values
			SKG.selectedClass = 0
			SKG_LFIPF_ClassChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTCLASS_TEXT"] )
			SKG_LFIPF_MainChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTMAIN_TEXT"] )
		end )


-- onload stuff:
	SKG_AddAlphaGroups( SKG.mainSelections )

-- OnLoad code

	SKG.classSelections.args = { }
	SKG.selectedClass = 0
	for i=1, #SKG_ClassNames do
		local t = {
			type = "execute",
			name = SKG_ClassNames[i],
			desc = SKG_ClassNames[i],
			func = function ( val )
				SKG.selectedClass = val
				SKG_LFIPF_ClassChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTCLASS_TEXT"] .. ": " .. SKG_ClassNames[val] )
				Dewdrop:Close()
			end,
			passValue = i
		}
		tinsert( SKG.classSelections.args, t )
	end
	SKG_LFIPF_ClassChooser:Enable()

-- Timer Frames for doing list insertions once every three seconds

SKG_PlayerTimerFrame = CreateFrame("Frame",SKG_PlayerTimerFrame,UIParent)
	SKG_PlayerTimerFrame.t = 0
	SKG_PlayerTimerFrame:SetScript("OnUpdate", function(self,elapsed)
		SKG_PlayerTimerFrame.t = SKG_PlayerTimerFrame.t + elapsed;
		if ( SKG_PlayerTimerFrame.t > 3 ) then 
			SKG_PlayerTimerFrame.t = 0
			-- This routine looks at current raid members, and if one is NOT
			-- present in the Player's list, it will send the Comm Event to add
			-- it. If there are no players in the list that are not members, it
			-- hides the frame again.
			local pList = SKG.db.profile.Players
			local found = 0
			for i = 1, #SKG.Raiders do
				if ( SKG.Raiders[i].mPID == 0 ) then
					-- this person is not in the list
					found = i
					break
				end
			end
			if ( found == 0 ) then
				-- There's noone left to add. Print out a message to that effect,
				-- and hide our TimerFrame again
				SKG_Print( SKGL["SKG_MSG_ALLPLAYERS_ADDED"] )
				SKG_PlayerTimerFrame:Hide()
			else
				-- We got one... pitch the event...
				local newTID = SKG_CreateTID( 0, 0, 0 )
				local Payload = {}
				Payload = { newTID, SKG.Raiders[found].name, SKG.Raiders[found].class }
				SKG_SendComm( "PLRAD", Payload, "GUILD", nil )
			end
		end
	end )
	SKG_PlayerTimerFrame:Hide()

SKG_ListTimerFrame = CreateFrame("Frame",SKG_ListTimerFrame,UIParent)
	SKG_ListTimerFrame.t = 0
	SKG_ListTimerFrame:SetScript("OnUpdate", function(self,elapsed)
		SKG_ListTimerFrame.t = SKG_ListTimerFrame.t + elapsed;
		if ( SKG_ListTimerFrame.t > 3 ) then 
			SKG_ListTimerFrame.t = 0
			-- This routine looks at current raid members, and if one is NOT
			-- present in the Player's list, it will send the Comm Event to add
			-- it. If there are no players in the list that are not members, it
			-- hides the frame again.
			local currList = SKG_ActiveListTitle:GetText()
			local db = SKG.db.profile
			local pList = db.Players
			local rList  = db.RollLists[currList]
			local found = 0
			for i = 1, #SKG.Raiders do
				currPID = SKG.Raiders[i].mPID
				local notfound = true
				for j = 1, #rList do
					if ( rList[j] == SKG.Raiders[i].mPID ) then
						notfound = false
						break
					end
				end
				if ( notfound == true ) then
					found = i
					break
				end
			end
			if ( found == 0 ) then
				-- There's noone left to add. Print out a message to that effect,
				-- and hide our TimerFrame again
				SKG_Print( SKGL["SKG_MSG_ALLPLAYERS_ADDED"] )
				SKG_ListTimerFrame:Hide()
			else
				local newTID = SKG_CreateTID( 0, 0, 0 )
				local Payload = {}
				local PID = SKG.Raiders[found].mPID
				if ( SKG.db.profile.Options.List.AutoAdd == true ) then
					local position = 0
					if ( #rList == 0 ) then
						position = 1
					else
						position = math.random( 1, #rList )
					end
					SKG_Debug( "autoadd position = " .. position )
					Payload = { newTID, currList, PID, position }
				else
					Payload = { newTID, currList, PID }
				end
				SKG_SendComm( "PLRIN", Payload, "GUILD", nil )
			end
		end
	end )
	SKG_ListTimerFrame:Hide()

--[[
************************************************************************
CHANGELOG:

Date : 10/25/08
	Initial attempt with Ace3 instead of LibRock. Probably will require
	major edits once. Added header/footer. 
************************************************************************
]]--
